/*
 * @(#)FilledButtonUIHandler.java
 *
 * $Date: 2009-11-17 20:56:29 -0600 (Tue, 17 Nov 2009) $
 *
 * Copyright (c) 2009 by Jeremy Wood.
 * All rights reserved.
 *
 * The copyright of this software is owned by Jeremy Wood. 
 * You may not use, copy or modify this software, except in  
 * accordance with the license agreement you entered into with  
 * Jeremy Wood. For details see accompanying license terms.
 * 
 * This software is probably, but not necessarily, discussed here:
 * http://javagraphics.blogspot.com/
 * 
 * And the latest version should be available here:
 * https://javagraphics.dev.java.net/
 */
package com.bric.plaf.demo;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Shape;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;

import com.bric.plaf.FilledButtonUI;
import com.bric.plaf.ShimmerUIEffect;
import com.bric.plaf.ZoomIconUIEffect;

public class FilledButtonUIHandler implements UIHandler {
	
	private static final String BUTTON_TEXT = "XYZ";
	private static Icon DEMO_ICON = new ImageIcon(UIDemo.class.getResource("vcard_add.png"));
	
	Vector buttons = new Vector();
	
	JCheckBox iconCheckBox = new JCheckBox("Show Icons",true);
	JCheckBox contentCheckBox = new JCheckBox("Paint Content",true);
	JCheckBox borderCheckBox = new JCheckBox("Paint Border",true);
	JCheckBox focusCheckBox = new JCheckBox("Paint Focus",true);
	JCheckBox textCheckBox = new JCheckBox("Show Text",true);
	JCheckBox blinkCheckBox = new JCheckBox("Blink Focus",false);
	JCheckBox shimmerCheckBox = new JCheckBox("Shimmer Effect",false);
	JCheckBox zoomCheckBox = new JCheckBox("Zoom Icon Effect",false);
	JCheckBox enabledCheckBox = new JCheckBox("Enabled",true);
	JComboBox componentTypes = new JComboBox();
	
	public FilledButtonUIHandler() {

		iconCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Icon icon = iconCheckBox.isSelected() ? DEMO_ICON : null;

				for(int a = 0; a<buttons.size(); a++) {
					((AbstractButton) buttons.get(a) ).setIcon(icon);
				}
				
				pack();
			}
		});
		textCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String text = textCheckBox.isSelected() ? BUTTON_TEXT : "";

				for(int a = 0; a<buttons.size(); a++) {
					((AbstractButton) buttons.get(a) ).setText(text);
				}

				pack();
			}
		});
		
		blinkCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boolean blinking = blinkCheckBox.isSelected();
				for(int a = 0; a<buttons.size(); a++) {
					((AbstractButton) buttons.get(a) ).putClientProperty("Focus.blink",
							new Boolean(blinking));
				}
			}
		});
		
		shimmerCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					if(shimmerCheckBox.isSelected()) {
						button.addMouseListener(ShimmerUIEffect.mouseListener);
					} else {
						button.removeMouseListener(ShimmerUIEffect.mouseListener);
					}
				}
			}
		});
		
		focusCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					button.setFocusPainted( focusCheckBox.isSelected() );
				}
			}
		});
		borderCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					button.setBorderPainted( borderCheckBox.isSelected() );
				}
			}
		});
		contentCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					button.setContentAreaFilled( contentCheckBox.isSelected() );
				}
			}
		});
		
		zoomCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					if(zoomCheckBox.isSelected()) {
						button.addActionListener(ZoomIconUIEffect.actionListener);
					} else {
						button.removeActionListener(ZoomIconUIEffect.actionListener);
					}
				}
			}
		});
		
		enabledCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(int a = 0; a<buttons.size(); a++) {
					AbstractButton button = ((AbstractButton) buttons.get(a) );
					button.setEnabled(enabledCheckBox.isSelected());
				}
			}
		});

		componentTypes.addItem(JButton.class);
		componentTypes.addItem(JToggleButton.class);

		componentTypes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateVisibility();
			}
		});

		componentTypes.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1L;

			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				Class type = (Class)value;
				String name = type.getName();
				name = name.substring(name.lastIndexOf('.')+1);
				
				return super.getListCellRendererComponent(list, name, index, isSelected,
						cellHasFocus);
			}
		});
		
		updateVisibility();
	}
	
	protected void updateVisibility() {
		Class type = (Class)componentTypes.getSelectedItem();
		if(type!=null) {
			for(int a = 0; a<buttons.size(); a++) {
				Component c = (Component)buttons.get(a);
				c.setVisible( c.getClass().equals(type) );
			}
		}
	}

	public JComponent[] getControls() {
		return new JComponent[] {
				iconCheckBox, contentCheckBox,
				borderCheckBox, focusCheckBox,
				textCheckBox, blinkCheckBox,
				shimmerCheckBox, zoomCheckBox,
				enabledCheckBox, componentTypes
		};
	}

	public JPanel makeDemoPanel(ComponentUI ui) {
		if((ui instanceof FilledButtonUI)==false)
			return null;
		FilledButtonUI fui = (FilledButtonUI)ui;
		
		JPanel motherPanel = new JPanel(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		JPanel segmentPanel = new JPanel(new GridBagLayout());
		String[] hPos = new String[] {"first", "middle", "last", "only"};
		String[] vPos = new String[] {"top", "middle", "bottom", "only"};
		
		c.weightx = 1; c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		
		for(int h = 0; h<hPos.length; h++) {
			for(int v = 0; v<vPos.length; v++) {
				String name = ui.getClass().getName();
				name = name.substring(name.lastIndexOf('.')+1);
				String toolTip = "this is a demo of \""+name+"\".";
				
				AbstractButton button1 = new JButton(BUTTON_TEXT);
				AbstractButton button2 = new JToggleButton(BUTTON_TEXT);
				
				for(int a = 0; a<2; a++) {
					AbstractButton button = (a==0) ? button1 : button2;
					button.setOpaque(false);
					button.setIcon(DEMO_ICON);
					button.putClientProperty("JButton.segmentHorizontalPosition", hPos[h]);
					button.putClientProperty("JButton.segmentVerticalPosition", vPos[v]);
					button.setUI( fui );
					button.setFont(UIManager.getFont("IconButton.font")); //miniature-ish
					buttons.add(button);
					button.setToolTipText(toolTip);
				}
				
				c.gridx = h; c.gridy = v;
				
				segmentPanel.add(button1,c);
				segmentPanel.add(button2,c);
			}
		}
		
		JPanel shapesPanel = new JPanel(new GridBagLayout());
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		c.weightx = 1; c.weighty = 1;
		c.fill = GridBagConstraints.NONE;
		
		for(int z = 0; z<3; z++) {

			AbstractButton button1 = new JButton(BUTTON_TEXT);
			AbstractButton button2 = new JToggleButton(BUTTON_TEXT);

			String name = ui.getClass().getName();
			name = name.substring(name.lastIndexOf('.')+1);
			String toolTip = "this is a demo of \""+name+"\"";
			
			Shape shape = null;
			if(z==0) {
				shape = new Ellipse2D.Float(0,0,20,20);
				toolTip = toolTip+" using an ellipse.";
			} else if(z==1) {
				GeneralPath diamond = new GeneralPath();
				diamond.moveTo(0, 0);
				diamond.lineTo(10, 10);
				diamond.lineTo(0, 20);
				diamond.lineTo(-10, 10);
				diamond.closePath();
				shape = diamond;
				toolTip = toolTip+" using a diamond.";
			} else {
				GeneralPath arrow = new GeneralPath();
				arrow.moveTo(0, -5);
				arrow.lineTo(20, -5);
				arrow.lineTo(20, -10);
				arrow.lineTo(30, 0);
				arrow.lineTo(20, 10);
				arrow.lineTo(20, 5);
				arrow.lineTo(0, 5);
				arrow.closePath();
				shape = arrow;
				toolTip = toolTip+" using an arrow.";
			}
			button1.putClientProperty(FilledButtonUI.SHAPE, shape);
			button2.putClientProperty(FilledButtonUI.SHAPE, shape);
			
			for(int a = 0; a<2; a++) {
				AbstractButton button = (a==0) ? button1 : button2;
				button.setOpaque(false);
				button.setIcon(DEMO_ICON);
				button.setUI( fui );
				button.setFont(UIManager.getFont("IconButton.font")); //miniature-ish
				button.setToolTipText(toolTip);
			}
			
			c.gridx = z; c.gridy = 0;
			
			shapesPanel.add(button1,c);
			shapesPanel.add(button2,c);
		}
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		c.weightx = 1; c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		motherPanel.add(segmentPanel, c);
		
		c.gridy++; c.weighty = 0; 
		c.fill = GridBagConstraints.HORIZONTAL;
		motherPanel.add(shapesPanel, c);
		
		segmentPanel.setOpaque(false);
		shapesPanel.setOpaque(false);
		motherPanel.setOpaque(false);
		return motherPanel;
	}
	
	protected void pack() {
		if(buttons.size()>0) {
			Window w = SwingUtilities.getWindowAncestor( (AbstractButton)buttons.get(0) );
			if(w!=null) {
				w.pack();
			}
		}
	}

	public void updateControls(JInternalFrame selectedFrame, ComponentUI ui,
			Vector components) {
		buttons.removeAllElements();
		for(int a = 0; a<components.size(); a++) {
			if(components.get(a) instanceof AbstractButton)
				buttons.add(components.get(a));
		}
		
	}

}
